var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Connector } from '@web3-react/types';
export class WalletConnect extends Connector {
    constructor(actions, options, connectEagerly = true) {
        super(actions);
        this.options = options;
        if (connectEagerly) {
            this.providerPromise = this.startListening(connectEagerly);
        }
    }
    startListening(connectEagerly) {
        return __awaiter(this, void 0, void 0, function* () {
            const WalletConnectProvider = yield import('@walletconnect/ethereum-provider').then((m) => { var _a; return (_a = m === null || m === void 0 ? void 0 : m.default) !== null && _a !== void 0 ? _a : m; });
            this.provider = new WalletConnectProvider(this.options);
            this.provider.on('disconnect', (error) => {
                this.actions.reportError(error);
            });
            this.provider.on('chainChanged', (chainId) => {
                this.actions.update({ chainId });
            });
            this.provider.on('accountsChanged', (accounts) => {
                this.actions.update({ accounts });
            });
            // silently attempt to eagerly connect
            if (connectEagerly && this.provider.connected) {
                Promise.all([
                    this.provider.request({ method: 'eth_chainId' }),
                    this.provider.request({ method: 'eth_accounts' }),
                ])
                    .then(([chainId, accounts]) => {
                    if (accounts.length > 0) {
                        this.actions.update({ chainId, accounts });
                    }
                })
                    .catch((error) => {
                    console.debug('Could not connect eagerly', error);
                });
            }
        });
    }
    activate() {
        return __awaiter(this, void 0, void 0, function* () {
            this.actions.startActivation();
            if (!this.providerPromise) {
                this.providerPromise = this.startListening(false);
            }
            yield this.providerPromise;
            // this.provider guaranteed to be defined now
            yield Promise.all([
                this.provider.request({ method: 'eth_chainId' }),
                this.provider.request({ method: 'eth_requestAccounts' }),
            ])
                .then(([chainId, accounts]) => {
                this.actions.update({ chainId, accounts });
            })
                .catch((error) => {
                this.actions.reportError(error);
            });
        });
    }
    deactivate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.provider) {
                yield this.provider.disconnect();
            }
        });
    }
}
